from os import path, listdir

path_tgui_libs = path.abspath(path.dirname(__file__))
path_tgui_libs_win32_x64 = path.join(path_tgui_libs, "win32-x64")

libs = {}

if path.exists(path_tgui_libs):
    for lib in listdir(path_tgui_libs):
        _path = path.join(path_tgui_libs, lib)
        if path.isfile(_path):
            if path.splitext(_path)[1] == ".dll":  # 判断文件扩展名是否为“.dll”
                libs[lib] = _path

"""from sys import platform

if platform == "win32":
    if path.exists(path_tgui_libs_win32_x64):
        for lib in listdir(path_tgui_libs_win32_x64):
            _path = path.join(path_tgui_libs_win32_x64, lib)
            if path.isfile(_path):
                if path.splitext(_path)[1] == ".dll":  # 判断文件扩展名是否为“.dll”
                    libs[lib] = _path

print(libs)"""


from .libs import libs
from ..core import NImport

tgui = NImport(libs["tgui.net.dll"])
SFML_Audio = NImport(libs["SFML.Audio.dll"])
SFML_Graphics = NImport(libs["SFML.Graphics.dll"])
SFML_System = NImport(libs["SFML.System.dll"])
SFML_Window = NImport(libs["SFML.Window.dll"])

from .gui import NTGui
from .renderwindow import NTRenderWindow
from .window import NTWindow, STYLE_CLOSE, STYLE_DEFAULT, STYLE_FULLSCREEN, STYLE_NONE, STYLE_RESIZE, STYLE_TITLEBAR


from ..base import NBase


from SFML.Window import Styles

STYLE_CLOSE = Styles.Close
STYLE_DEFAULT = Styles.Default
STYLE_FULLSCREEN = Styles.Fullscreen
STYLE_NONE = getattr(Styles, "None")
STYLE_RESIZE = Styles.Resize
STYLE_TITLEBAR = Styles.Titlebar


class NTWindow(NBase):
    from SFML.Window import Window

    _type = Window

    def __init__(self, *args, width: int = 400, height: int = 300, title: str = "TGUI", style=STYLE_DEFAULT, **kwargs):
        from SFML.Window import VideoMode
        super().__init__(VideoMode(width, height), title, style)

        self.new_closed(self._close)

    def _close(self, s=None, e=None):
        self.close()

    def close(self):
        self._.Close()

    def dispatch_events(self):
        self._.DispatchEvents()

    def display(self):
        self._.Display()

    @property
    def isopen(self):
        return self._.IsOpen

    def mainloop(self):
        while self.isopen:
            self.dispatch_events()
            self.display()

    def new_closed(self, func):
        self._.Closed += lambda s, e: func(s, e)

    def remove_closed(self, func):
        self._.Closed -= func

    def title(self, text: str = None):
        if text:
            self._.SetTitle(text)


from .window import NTWindow


class NTRenderWindow(NTWindow):
    from SFML.Graphics import RenderWindow

    _type = RenderWindow

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        from .gui import NTGui

        self.gui = NTGui(self)

    def clear(self):
        self._.Clear()

    def mainloop(self):
        while self.isopen:
            self.dispatch_events()
            self.clear()
            self.gui.draw()
            self.display()


from ..base import NBase


class NTGui(NBase):
    from TGUI import Gui

    _type = Gui

    from .renderwindow import NTRenderWindow

    def __init__(self, renderwindow: NTRenderWindow):
        super().__init__(renderwindow._)

    def add(self, widget: NBase):
        self._.Add(widget)

    def draw(self):
        self._.Draw()
